/*-------------- Telecommunications & Signal Processing Lab ---------------
                             McGill University

Routine:
  AFILE *AFwrWVhead (FILE *fp, int Format, long int Nframe, long int Nchan,
                     double Sfreq)

Purpose:
  Write header information to a RIFF WAVE file

Description:
  This routine opens and writes header information to a RIFF WAVE format audio
  file.

  RIFF WAVE file:
   Offset Length Type    Contents
      0     4    char   File identifier ("RIFF")
      4     4    int    RIFF chunk length
      8     4    char   File identifier ("WAVE")
     12     4    char   Format chunk identifier ("fmt ")
     16     4    int    Format chunk length
     20     2    int    Audio data type
     22     2    int    Number of interleaved channels
     24     4    int    Sample rate
     28     4    int    Average bytes/sec
     32     2    int    Block align
     34     2    int    Data word length (bits)
    ...   ...    ...    ...
      D     4    char   Data chunk identifier ("data")
    D+4     4    int    Data chunk length
    D+8    ...   ...    Audio data

Parameters:
  <-  AFILE AFwrWVhead
      Audio file pointer for the audio file.  In case of error, the audio file
      pointer is set to NULL.
   -> FILE *fp
      File pointer for the audio file
   -> int Format
      Audio file data format code: FD_MULAW8, FD_ALAW8, FD_UINT8, etc.
   -> long int Nframe
      Number of sample frames.  This value is used to set the file header
      information for non-random access files for which the header cannot be
      updated when the file is closed.  If this value is AF_NFRAME_UNDEF, the
      file must be random access.
   -> long int Nchan
      Number of channels
   -> double Sfreq
      Sampling frequency

Author / revision:
  P. Kabal  Copyright (C) 1999
  $Revision: 1.48 $  $Date: 1999/06/04 20:07:54 $

-------------------------------------------------------------------------*/

static char rcsid [] = "$Id: AFwrWVhead.c 1.48 1999/06/04 AFsp-v5r0 $";

#include <assert.h>
#include <setjmp.h>
#include <string.h>

#include <libtsp.h>
#include <libtsp/nucleus.h>
#include <libtsp/AFheader.h>
#include <libtsp/AFmsg.h>
#define AF_DATA_LENGTHS
#include <libtsp/AFpar.h>
#include <libtsp/WVpar.h>

#define ICEILV(n,m)	(((n) + ((m) - 1)) / (m))	/* int n,m >= 0 */
#define RNDUPV(n,m)	((m) * ICEILV (n, m))		/* Round up */

#define MCOPY(src,dest)		memcpy ((void *) (dest), \
					(const void *) (src), sizeof (dest))
#define WRPAD(fp,size,align) \
     AFwriteHead (fp, NULL, 1, (int) (RNDUPV(size, align) - (size)), \
		  DS_NATIVE);

#define ALIGN		2	/* Chunks padded out to a multiple of ALIGN */

/* setjmp / longjmp environment */
extern jmp_buf AFW_JMPENV;

static void
AF_setFMT (struct WV_PCMfmt *fmt, int Format, long int Nchan, double Sfreq);
static int
AF_wrFMT (FILE *fp, const struct WV_PCMfmt *fmt);


AFILE *
AFwrWVhead (FILE *fp, int Format, long int Nframe, long int Nchan,
	    double Sfreq)

{
  AFILE *AFp;
  int Lw;
  long int size, Ldata;
  struct {
    struct WV_CkPreamb riff_p;
    char waveid[4];
    struct WV_PCMfmt fmt;
    struct WV_CkPreamb data_p;
  } Fhead;
  struct AF_write AFw;

/* Set the long jump environment; on error return a NULL */
  if (setjmp (AFW_JMPENV))
    return NULL;	/* Return from a header write error */

/* Set up the encoding parameters */
  switch (Format) {
  case FD_INT16:
    AFw.DFormat.ScaleF = 1./WV_SF_PCM16;
    break;
  case FD_INT24:
    AFw.DFormat.ScaleF = 1./WV_SF_PCM24;
    break;
  case FD_INT32:
    AFw.DFormat.ScaleF = 1./WV_SF_PCM32;
    break;
  case FD_UINT8:
    AFw.DFormat.ScaleF = 1./WV_SF_PCM8;
    break;
  case FD_MULAW8:
    AFw.DFormat.ScaleF = 1./WV_SF_MULAW;
    break;
  case FD_ALAW8:
    AFw.DFormat.ScaleF = 1./WV_SF_ALAW;
    break;
  case FD_FLOAT32:
    AFw.DFormat.ScaleF = 1./WV_SF_FLOAT32;
    if (! UTcheckIEEE ())
      UTwarn ("AFwrWVhead - %s", AFM_WV_NoIEEE);
    break;
  case FD_FLOAT64:
    AFw.DFormat.ScaleF = 1./WV_SF_FLOAT64;
    if (! UTcheckIEEE ())
      UTwarn ("AFwrWVhead - %s", AFM_WV_NoIEEE);
    break;
  default:
    UTwarn ("AFwrWVhead - %s", AFM_WV_UnsData);
    return NULL;
  }
  AFw.DFormat.Format = Format;
  AFw.DFormat.Swapb = DS_EL;

  AFw.Sfreq = Sfreq;
  AFw.Nchan = Nchan;

/* Set up the fixed header parameters */
  Lw = AF_DL[Format];
  if (FLseekable (fp))
    Ldata = 0L;
  else if (Nframe != AF_NFRAME_UNDEF)
    Ldata = Nframe * Nchan * Lw;
  else {
    UTwarn ("AFwrWVhead - %s", AFM_WV_WRAccess);
    return NULL;
  }

  /* riff chunk */
  MCOPY (FM_RIFF, Fhead.riff_p.ckid);
  /* defer filling in the chunk size */ 
  MCOPY (FM_WAVE, Fhead.waveid);

  /* fmt chunk */
  AF_setFMT (&Fhead.fmt, Format, Nchan, Sfreq);

  /* data chunk */
  MCOPY ("data", Fhead.data_p.ckid);
  Fhead.data_p.cksize = (uint4_t) Ldata;

  /* riff chunk size */
  size = 4 + 8 + RNDUPV(Fhead.fmt.cksize, ALIGN);
  size += 8 + RNDUPV(Fhead.data_p.cksize, ALIGN);
  Fhead.riff_p.cksize = (uint4_t) size;

/* Write out the header */
  WHEAD_S (fp, Fhead.riff_p.ckid);
  WHEAD_V (fp, Fhead.riff_p.cksize, DS_EL);
  WHEAD_S (fp, Fhead.waveid);

  AF_wrFMT (fp, &Fhead.fmt);

  WHEAD_S (fp, Fhead.data_p.ckid);
  WHEAD_V (fp, Fhead.data_p.cksize, DS_EL);

/* Header information records */
  AFw.Hinfo = *(AFgenHinfo (Sfreq));

/* Set the parameters for file access */
  AFp = AFsetWrite (fp, FT_WAVE, &AFw);

  return AFp;
}

/* Fill in the fmt chunk */


static void
AF_setFMT (struct WV_PCMfmt *fmt, int Format, long int Nchan, double Sfreq)

{
  int Lw;

  MCOPY ("fmt ", fmt->ckid);
  fmt->cksize = sizeof (fmt->FormatTag) + sizeof (fmt->Channels)
    + sizeof (fmt->SamplesPerSec) + sizeof (fmt->AvgBytesPerSec)
    + sizeof (fmt->BlockAlign) + sizeof (fmt->BitsPerSample);

  switch (Format) {
  case FD_UINT8:
  case FD_INT16:
  case FD_INT24:
  case FD_INT32:
    fmt->FormatTag = WAVE_FORMAT_PCM;
    break;
  case FD_MULAW8:
    fmt->FormatTag = WAVE_FORMAT_MULAW;
    break;
  case FD_ALAW8:
    fmt->FormatTag = WAVE_FORMAT_ALAW;
    break;
  case FD_FLOAT32:
  case FD_FLOAT64:
    fmt->FormatTag = WAVE_FORMAT_IEEE_FLOAT;
    break;
  default:
    assert (0);
  }

  Lw = AF_DL[Format];
  fmt->Channels = (uint2_t) Nchan;
  fmt->SamplesPerSec = (uint4_t) (Sfreq + 0.5);	/* Rounding */
  fmt->AvgBytesPerSec = (uint4_t) (fmt->SamplesPerSec * Nchan * Lw);
  fmt->BlockAlign = (uint2_t) (Lw * Nchan);
  fmt->BitsPerSample = (uint2_t) (8 * Lw);

  return;
}

/* Write the format chunk */


static int
AF_wrFMT (FILE *fp, const struct WV_PCMfmt *fmt)

{
  int offs;

  offs = WHEAD_S (fp, fmt->ckid);
  offs += WHEAD_V (fp, fmt->cksize, DS_EL);
  offs += WHEAD_V (fp, fmt->FormatTag, DS_EL);
  offs += WHEAD_V (fp, fmt->Channels, DS_EL);
  offs += WHEAD_V (fp, fmt->SamplesPerSec, DS_EL);
  offs += WHEAD_V (fp, fmt->AvgBytesPerSec, DS_EL);
  offs += WHEAD_V (fp, fmt->BlockAlign, DS_EL);
  offs += WHEAD_V (fp, fmt->BitsPerSample, DS_EL);

  assert (8 + (int) fmt->cksize == offs);

  return offs;
}
